/*
 * This script moves the current selection so that the track
 * end selected by user aligns to another track without
 * changing the track angle. The original track is removed.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-05
 */

include("../QCTrack.js");
include("../../Modify/Transform.js");

function Align(guiAction) {
    Transform.call(this, guiAction);

    this.lineSource = undefined;
    this.lineDest = undefined;
}

Align.prototype = new Transform();

Align.includeBasePath = includeBasePath;

Align.State = {
    SettingLineSource : 0,
    SettingLineDest : 1,
    SettingLineAlign : 2
};

Align.prototype.beginEvent = function() {
    Transform.prototype.beginEvent.call(this);

    if (!this.verifySelection()) {
        return;
    }

    this.copies = 0; //delete original
    this.useCurrentAttributes = false; //don't change current attributes
    this.setState(Align.State.SettingLineSource);
    this.scaling = new RVector(5000, 5000);
};

Align.prototype.setState = function(state) {
    Transform.prototype.setState.call(this, state);

    this.getDocumentInterface().setClickMode(RAction.PickEntity);
    this.setCrosshairCursor();

    var appWin = RMainWindowQt.getMainWindow();
    var info;
    switch (this.state) {
    case Align.State.SettingLineSource:
      this.lineSource = undefined;
      this.lineDest = undefined;
      this.lineAlign = undefined;
      info = qsTr("Select end of track to move");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trCancel);
      break;
    case Align.State.SettingLineDest:
      this.lineDest = undefined;
      this.lineAlign = undefined;
      info = qsTr("Select destination track end");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trBack);
      break;
    case Align.State.SettingLineAlign:
      this.lineAlign = undefined;
      info = qsTr("Select alignment track end");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trBack);
      break;
    }
};

Align.prototype.escapeEvent = function() {
    switch (this.state) {
    case Align.State.SettingLineSource:
      Transform.prototype.escapeEvent.call(this);
      break;
    case Align.State.SettingLineDest:
      this.setState(Align.State.SettingLineSource);
      break;
    case Align.State.SettingLineAlign:
      this.setState(Align.State.SettingLineDest);
      break;
    }
};

Align.prototype.pickEntity = function(event, preview) {
    var di = this.getDocumentInterface();
    var document = this.getDocument();
    var entityId = event.getEntityId();
    var entity = document.queryEntity(entityId);
    var pos = event.getModelPosition();
    var op;

    switch (this.state) {
    case Align.State.SettingLineSource:
      if (isNull(entity)) {
        this.lineSource = undefined;
        this.lineDest = undefined;
        this.lineAlign = undefined;
        this.raySource = undefined;
        this.rayDest = undefined;
        this.rayAlign = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineSource = getClosestTrackEnd(document, entity, pos, true);
      if (this.lineSource == undefined) {
        this.raySource = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.raySource = new RLine(this.lineSource.getStartPoint(), this.lineSource.getEndPoint());
      this.raySource.rotate(Math.PI / 2, this.lineSource.getMiddlePoints()[0]);
      this.raySource.scale(this.scaling,  this.lineSource.getMiddlePoints()[0]);
      if (preview) {
        this.updatePreview();
      }
      else {
        this.setState(Align.State.SettingLineDest);
      }
      break;
    case Align.State.SettingLineDest:
      if (isNull(entity)) {
        this.lineDest = undefined;
        this.lineAlign = undefined;
        this.rayDest = undefined;
        this.rayAlign = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineDest = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineDest == undefined) {
        this.rayDest = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.rayDest = new RLine(this.lineDest.getStartPoint(), this.lineDest.getEndPoint());
      this.rayDest.rotate(Math.PI / 2, this.lineDest.getMiddlePoints()[0]);
      this.rayDest.scale(this.scaling,  this.lineDest.getMiddlePoints()[0]);
      if (preview) {
        this.updatePreview();
      }
      else {
        this.setState(Align.State.SettingLineAlign);
      }
      break;
    case Align.State.SettingLineAlign:
      if (isNull(entity)) {
        this.lineAlign = undefined;
        this.rayAlign = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineAlign = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineAlign == undefined) {
        this.rayAlign = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.rayAlign = new RLine(this.lineAlign.getStartPoint(), this.lineAlign.getEndPoint());
      this.rayAlign.rotate(Math.PI / 2, this.lineAlign.getMiddlePoints()[0]);
      this.rayAlign.scale(this.scaling,  this.lineAlign.getMiddlePoints()[0]);
      if (preview) {
        this.updatePreview();
      }
      else {
        op = this.getOperation(false);
        if (op == undefined) break;
        di.applyOperation(op);
        this.setState(-1);
        this.terminate();
      }
      break;
    }
};

Align.prototype.getOperation = function(preview) {
    if (this.raySource == undefined || this.rayDest == undefined || this.rayAlign == undefined) {
      return undefined;
    }

    var isctSrcAln, isctDstAln;
    isctSrcAln = this.raySource.getIntersectionPoints(this.rayAlign);
    isctDstAln = this.rayDest.getIntersectionPoints(this.rayAlign);
    if (isctSrcAln.length != 1 || isctDstAln.length != 1) { //intersections are not both valid
      return undefined;
    }

    var moveX, moveY;
    moveX = isctDstAln[0].getX() - isctSrcAln[0].getX();
    moveY = isctDstAln[0].getY() - isctSrcAln[0].getY();
    this.moveBy = new RVector(moveX, moveY);
    if (!preview) {
      EAction.handleUserMessage(qsTr("move by ") + moveX + " / " + moveY);
    }
    return Transform.prototype.getOperation.call(this, preview);
};

Align.prototype.transform = function(entity, k, op) {
    entity.move(this.moveBy);
    op.addObject(entity, this.useCurrentAttributes, false);
};

Align.prototype.getAuxPreview = function() {
    var ret = undefined;
    if (isLineEntity(this.lineSource)) {
      ret = new Array();
      addAuxPreviewObjects(this.lineSource, ret);
      ret.push(this.raySource);
      if (isLineEntity(this.lineDest)) {
        addAuxPreviewObjects(this.lineDest, ret);
        ret.push(this.rayDest);
        if (isLineEntity(this.lineAlign)) {
          addAuxPreviewObjects(this.lineAlign, ret);
          ret.push(this.rayAlign);
        }
      }
    }
    return ret;
};

Align.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.Align, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setRequiresSelection(true);
    action.setScriptFile(basePath + "/Align.js");
    action.setIcon(basePath + "/Align.svg");
    action.setDefaultShortcut(new QKeySequence("q,a"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.Align);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
    var appWin = EAction.getMainWindow();
    appWin.addSelectionListener(action);
};

