/*
 * This script creates a track with parameters according to the user dialog
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-18
 */

//http://www.ribbonsoft.com/doc/qcad/3.0/developer/class_r_entity.html

include("../QCTrack.js");
include("../../WidgetFactory.js");

function Create(guiAction) {
    QCTrack.call(this, guiAction);
}

Create.prototype = new EAction();
Create.includeBasePath = includeBasePath;

Create.prototype.beginEvent = function() {
    EAction.prototype.beginEvent.call(this);
    var document = EAction.getDocument();
    var di = EAction.getDocumentInterface();

    //let the user enter the desired parameters
    var dialog = WidgetFactory.createWidget(
                   "scripts/QCTrack/Create",
                   "CreateDialog.ui");
    WidgetFactory.restoreState(dialog);
    var widgets = getWidgets(dialog);
    if (document.queryCurrentBlock().getName() != RBlock.modelSpaceName) {
      widgets["createInsideBlock"].checked = false;
      widgets["displayName"].setFocus();
    }
    else {
      widgets["createInsideBlock"].checked = true;
      widgets["blockName"].setFocus();
    }
    this.okButton = widgets["buttonBox"].button(QDialogButtonBox.Ok);
    widgets["buttonBox"].button(QDialogButtonBox.RestoreDefaults)["clicked()"].connect(this, function() { WidgetFactory.restoreState(dialog, null, null, true); });
    widgets["displayNameSize"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0.001); });
    widgets["cost"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["trackDistance"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0.001); });
    widgets["bedSize"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["swellDistance"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["swellWidth"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["swellLength"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["straight"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["radius"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    widgets["angle"]["valueChanged(double,QString)"].connect(this, function(value, error) { this.validateEdit(value, error, 0); });
    if (!dialog.exec()) { //the user aborted
        dialog.destroy();
        this.terminate();
        return;
    }
    WidgetFactory.saveState(dialog);

    //get and check given parameters
    var insideBlock = widgets["createInsideBlock"].checked;
    var blockName = widgets["blockName"].text;
    var displayName = widgets["displayName"].text;
    var displayNameSize = widgets["displayNameSize"].getValue();
    var cost = Math.round(widgets["cost"].getValue() * 100);
    var trackDistance = widgets["trackDistance"].getValue();
    var bedSize = widgets["bedSize"].getValue();
    var swellDistance = widgets["swellDistance"].getValue();
    var swellWidth = widgets["swellWidth"].getValue();
    var swellLength = widgets["swellLength"].getValue();
    var straight = widgets["straight"].getValue();
    var radius = widgets["radius"].getValue();
    var angle = widgets["angle"].getValue();
    var rightNotLeft = widgets["curveRight"].checked;
    if (insideBlock && blockName == "") {
      EAction.handleUserMessage(qsTr("block name must not be empty if creating in block"));
      dialog.destroy();
      (new QMessageBox(null, qsTr("Create Track"), qsTr("block name must not be empty if creating in block, did not create anything"), QMessageBox.Ok, null, Qt.Dialog)).exec();
      this.terminate();
      return;
    }
    if (insideBlock && document.hasBlock(blockName)) {
      EAction.handleUserMessage(qsTr("block name already existing"));
      dialog.destroy();
      (new QMessageBox(null, qsTr("Create Track"), qsTr("block name already existing, did not change anything"), QMessageBox.Ok, null, Qt.Dialog)).exec();
      this.terminate();
      return;
    }
    if ((straight != 0 && (radius != 0 || angle != 0)) || (straight == 0 && radius == 0 && angle == 0)) {
      EAction.handleUserMessage(qsTr("either straight or curve (one set has to be greater zero, set other values to zero)"));
      dialog.destroy();
      (new QMessageBox(null, qsTr("Create Track"), qsTr("either straight or curve (one set has to be greater zero, set other values to zero), did not create anything"), QMessageBox.Ok, null, Qt.Dialog)).exec();
      this.terminate();
      return;
    }
    var oldLayerId = document.getCurrentLayerId();

    //check required layers
    checkLayerExistence(document, di, "TR_MIDS", "black");
    checkLayerExistence(document, di, "TR_PLAN", "black");
    checkLayerExistence(document, di, "TR_ENDS", "black");
    checkLayerExistence(document, di, "TR_NMBRS", "black");
    if (swellWidth > 0 && swellLength > 0 && swellDistance > 0) {
      checkLayerExistence(document, di, "TR_SWELLS", "orange");
    }
    if (bedSize > 0) {
      checkLayerExistence(document, di, "TR_BEDS", "grey");
    }
    if (cost > 0) {
      checkLayerExistence(document, di, "TR_COST", "red");
    }

    //create track with given parameters
    //--prepare
    var operation;
    if (insideBlock) {
      EAction.handleUserMessage(qsTr("creating ")+blockName);
      var block = new RBlock(document, blockName, new RVector(0,0));
      operation = new RAddObjectOperation(block);
      di.applyOperation(operation);
      di.setCurrentBlock(blockName);
    }
    else {
      EAction.handleUserMessage(qsTr("creating track"));
    }
    //--create entities
    if (cost > 0) {
      di.setCurrentColor(new RColor(RColor.ByLayer));
      di.setCurrentLayer("TR_COST");
      operation = new RAddObjectsOperation();
      p1 = new RVector(0, 0);
      text = new RTextEntity(document, new RTextData(
        p1,                  // position
        p1,                  // alignment point
        trackDistance / 4,   // height
        2.0,                 // text width (ignored for now)
        RS.VAlignMiddle,     // alignments
        RS.HAlignLeft,
        RS.LeftToRight,
        RS.Exact,
        1.0,                 // line spacing factor
        "" + cost,    // the text
        "Arial",      // font
        false,        // bold
        false,        // italic
        0.0,          // angle
        false         // simple text without formatting
      ));
      operation.addObject(text);
      di.applyOperation(operation);
    }
    if (straight > 0) { //create straight track
      var p1, p2, p3, p4, line, text;
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_MIDS");
      operation = new RAddObjectsOperation();
      p1 = new RVector(0, 0);
      p2 = new RVector(straight, 0);
      line = new RLineEntity(document, new RLineData(p1, p2));
      operation.addObject(line);
      di.applyOperation(operation);
      operation = new RAddObjectsOperation();
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_ENDS");
      p1 = new RVector(0, trackDistance / 2);
      p2 = new RVector(0, -trackDistance / 2);
      p3 = new RVector(straight, -trackDistance / 2);
      p4 = new RVector(straight, trackDistance / 2);
      line = new RLineEntity(document, new RLineData(p1, p2));
      operation.addObject(line);
      line = new RLineEntity(document, new RLineData(p3, p4));
      operation.addObject(line);
      di.applyOperation(operation);
      if (bedSize > 0) {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_BEDS");
        operation = new RAddObjectsOperation();
        p1 = new RVector(0, bedSize / 2);
        p2 = new RVector(0, -bedSize / 2);
        p3 = new RVector(straight, -bedSize / 2);
        p4 = new RVector(straight, bedSize / 2);
        addRectangle(document, operation, p1, p2, p3, p4);
        di.applyOperation(operation);
      }
      if (swellWidth > 0 && swellLength > 0 && swellDistance > 0) {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_SWELLS");
        operation = new RAddObjectsOperation();
        var i;
        var count = Math.floor((straight - swellWidth) / swellDistance) + 1;
        var baseOffset = (straight - (count - 1) * swellDistance - swellWidth) / 2;
        for (i = 0; i < count; i++) {
          var base = baseOffset + swellDistance * i;
          p1 = new RVector(base, swellLength / 2);
          p2 = new RVector(base, -swellLength / 2);
          p3 = new RVector(base + swellWidth, -swellLength / 2);
          p4 = new RVector(base + swellWidth, swellLength / 2);
          addHatch(document, operation, p1, p2, p3, p4);
        }
        di.applyOperation(operation);
      }
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_PLAN");
      operation = new RAddObjectsOperation();
      p1 = new RVector(0, trackDistance / 2);
      p2 = new RVector(0, -trackDistance / 2);
      p3 = new RVector(straight, -trackDistance / 2);
      p4 = new RVector(straight, trackDistance / 2);
      addRectangle(document, operation, p1, p2, p3, p4);
      di.applyOperation(operation);
      if (displayNameSize > 0 && displayName != "") {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_NMBRS");
        operation = new RAddObjectsOperation();
        p1 = new RVector(straight / 2, 0);
        text = new RTextEntity(document, new RTextData(
          p1,                  // position
          p1,                  // alignment point
          displayNameSize,     // height
          2.0,                 // text width (ignored for now)
          RS.VAlignMiddle,     // alignments
          RS.HAlignCenter,
          RS.LeftToRight,
          RS.Exact,
          1.0,                 // line spacing factor
          displayName,         // the text
          "Arial",      // font
          false,        // bold
          false,        // italic
          0.0,          // angle
          false         // simple text without formatting
        ));
        operation.addObject(text);
        di.applyOperation(operation);
      }
    }
    else { //create curved track
      var center, p1, p2, p3, p4, line, arc, text;
      //angle = RMath.deg2rad(angle); //not needed as RMathLineEdit supports direct angle value
      var startAngle =  rightNotLeft ? Math.PI / 2 - angle : 3 * Math.PI / 2;
      var endAngle =  rightNotLeft ? Math.PI / 2 : 3 * Math.PI / 2 + angle;
      var rotAngle = rightNotLeft ? -angle : angle;
      center = rightNotLeft ? new RVector(0, -radius) : new RVector(0, radius);
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_MIDS");
      operation = new RAddObjectsOperation();
      arc = new RArcEntity(document, new RArcData(center, radius, startAngle, endAngle, false));
      operation.addObject(arc);
      di.applyOperation(operation);
      operation = new RAddObjectsOperation();
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_ENDS");
      p1 = new RVector(0, trackDistance / 2);
      p2 = new RVector(0, -trackDistance / 2);
      line = new RLineEntity(document, new RLineData(p1, p2));
      operation.addObject(line);
      line = new RLineEntity(document, new RLineData(p2, p1));
      line.rotate(rotAngle, center);
      operation.addObject(line);
      di.applyOperation(operation);
      if (bedSize > 0) {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_BEDS");
        operation = new RAddObjectsOperation();
        arc = new RArcEntity(document, new RArcData(center, radius - bedSize / 2, startAngle, endAngle, false));
        operation.addObject(arc);
        arc = new RArcEntity(document, new RArcData(center, radius + bedSize / 2, startAngle, endAngle, false));
        operation.addObject(arc);
        p1 = new RVector(0, bedSize / 2);
        p2 = new RVector(0, -bedSize / 2);
        line = new RLineEntity(document, new RLineData(p1, p2));
        operation.addObject(line);
        line = new RLineEntity(document, new RLineData(p2, p1));
        line.rotate(rotAngle, center);
        operation.addObject(line);
        di.applyOperation(operation);
      }
      if (swellWidth > 0 && swellLength > 0 && swellDistance > 0) {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_SWELLS");
        operation = new RAddObjectsOperation();
        var i;
        var arclen = radius * angle;
        var count = Math.floor((arclen - swellWidth) / swellDistance) + 1;
        var baseAngle = (rotAngle / count * (swellDistance - swellWidth) / swellDistance) / 2;
        var angleDiff = rotAngle / count;
        for (i = 0; i < count; i++) {
          var curAngle = baseAngle + angleDiff * i;
          p1 = new RVector(0, swellLength / 2);
          p2 = new RVector(0, -swellLength / 2);
          p3 = new RVector(swellWidth, -swellLength / 2);
          p4 = new RVector(swellWidth, swellLength / 2);
          p1.rotate(curAngle, center);
          p2.rotate(curAngle, center);
          p3.rotate(curAngle, center);
          p4.rotate(curAngle, center);
          addHatch(document, operation, p1, p2, p3, p4);
        }
        di.applyOperation(operation);
      }
      di.setCurrentColor(new RColor(RColor.ByBlock));
      di.setCurrentLayer("TR_PLAN");
      operation = new RAddObjectsOperation();
      arc = new RArcEntity(document, new RArcData(center, radius - trackDistance / 2, startAngle, endAngle, false));
      operation.addObject(arc);
      arc = new RArcEntity(document, new RArcData(center, radius + trackDistance / 2, startAngle, endAngle, false));
      operation.addObject(arc);
      p1 = new RVector(0, trackDistance / 2);
      p2 = new RVector(0, -trackDistance / 2);
      line = new RLineEntity(document, new RLineData(p1, p2));
      operation.addObject(line);
      line = new RLineEntity(document, new RLineData(p2, p1));
      line.rotate(rotAngle, center);
      operation.addObject(line);
      di.applyOperation(operation);
      if (displayNameSize > 0 && displayName != "") {
        di.setCurrentColor(new RColor(RColor.ByLayer));
        di.setCurrentLayer("TR_NMBRS");
        operation = new RAddObjectsOperation();
        p1 = new RVector(0, 0);
        text = new RTextEntity(document, new RTextData(
          p1,                  // position
          p1,                  // alignment point
          displayNameSize,     // height
          2.0,                 // text width (ignored for now)
          RS.VAlignMiddle,     // alignments
          RS.HAlignCenter,
          RS.LeftToRight,
          RS.Exact,
          1.0,                 // line spacing factor
          displayName,         // the text
          "Arial",      // font
          false,        // bold
          false,        // italic
          0.0,          // angle
          false         // simple text without formatting
        ));
        text.rotate(rotAngle / 2, center);
        operation.addObject(text);
        di.applyOperation(operation);
      }
    }
    //--cleanup
    document.setCurrentLayer(oldLayerId);
    di.setCurrentLayer(document.queryLayer(oldLayerId).getName());
    if (insideBlock) {
      di.setCurrentBlock(RBlock.modelSpaceName);
    }
    EAction.handleUserMessage(qsTr("track creating done"));
    dialog.destroy();
    this.terminate();
};

Create.prototype.validateEdit = function(value, error, minimum) {
    var valid = error == "" && value >= minimum;
//    EAction.handleUserMessage("called, value == " + value + ", error == " + error + ", min == " + minimum + ", valid == " + valid);
    this.okButton.setEnabled(valid);
};

Create.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.Create, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setScriptFile(basePath + "/Create.js");
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.Create);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
};

