/*
 * This script displays some information about the distance
 * between two track ends.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-15
 */

//http://www.ribbonsoft.com/doc/qcad/3.0/developer/class_r_entity.html

include("../QCTrack.js");

function DistanceInfo(guiAction) {
    QCTrack.call(this, guiAction);
}

DistanceInfo.prototype = new EAction();
DistanceInfo.includeBasePath = includeBasePath;

DistanceInfo.State = {
    SettingLineStart : 0,
    SettingLineEnd : 1
};

DistanceInfo.prototype.beginEvent = function() {
    EAction.prototype.beginEvent.call(this);
    this.setState(DistanceInfo.State.SettingLineStart);
};

DistanceInfo.prototype.setState = function(state) {
    EAction.prototype.setState.call(this, state);

    this.getDocumentInterface().setClickMode(RAction.PickEntity);
    this.setCrosshairCursor();

    var appWin = RMainWindowQt.getMainWindow();
    var info;
    switch (this.state) {
    case DistanceInfo.State.SettingLineStart:
      this.lineSource = undefined;
      this.lineDest = undefined;
      info = qsTr("Select first track end");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trCancel);
      break;
    case DistanceInfo.State.SettingLineEnd:
      this.lineDest = undefined;
      info = qsTr("Select second track end");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trBack);
      break;
    }
};

DistanceInfo.prototype.escapeEvent = function() {
    switch (this.state) {
    case DistanceInfo.State.SettingLineStart:
      EAction.prototype.escapeEvent.call(this);
      break;
    case DistanceInfo.State.SettingLineEnd:
      this.setState(DistanceInfo.State.SettingLineStart);
      break;
    }
};

DistanceInfo.prototype.pickEntity = function(event, preview) {
    var di = this.getDocumentInterface();
    var document = this.getDocument();
    var entityId = event.getEntityId();
    var entity = document.queryEntity(entityId);
    var pos = event.getModelPosition();
    var op;

    switch (this.state) {
    case DistanceInfo.State.SettingLineStart:
      if (isNull(entity)) {
        this.lineStart = undefined;
        this.middleStart = undefined;
        this.lineEnd = undefined;
        this.middleEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineStart = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineStart == undefined) {
        this.middleStart = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.middleStart = this.lineStart.getMiddlePoints()[0];
      if (preview) {
        this.updatePreview();
      }
      else {
        this.setState(DistanceInfo.State.SettingLineEnd);
      }
      break;
    case DistanceInfo.State.SettingLineEnd:
      if (isNull(entity)) {
        this.lineEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineEnd = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineEnd == undefined) {
        this.middleEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.middleEnd = this.lineEnd.getMiddlePoints()[0];
      if (preview) {
        this.updatePreview();
      }
      else {
        EAction.handleUserMessage(qsTr("info from ")
          + this.middleStart.getX().toFixed(1) + " / " + this.middleStart.getY().toFixed(1) + qsTr(" at ")
          + RMath.rad2deg(this.lineStart.getAngle()).toFixed(2) + qsTr("° to ")
          + this.middleEnd.getX().toFixed(1) + " / " + this.middleEnd.getY().toFixed(1) + qsTr(" at ")
          + RMath.rad2deg(this.lineEnd.getAngle()).toFixed(2) + qsTr("°"));
        var angleDiff = getNormalizedAngle(this.lineEnd.getAngle() - this.lineStart.getAngle(), Math.PI * 2, RMath.deg2rad(0.001));
        var middleDiff = this.middleEnd.getDistanceTo(this.middleStart);
        EAction.handleUserMessage(qsTr("direct distance: ") + middleDiff.toFixed(1) + qsTr(", track end rotated by ")
          + RMath.rad2deg(angleDiff).toFixed(2) + qsTr("°"));
        if (angleDiff >= Math.PI) {
          angleDiff -= Math.PI;
        }
        if (angleDiff < RMath.deg2rad(2)) { //seems to be straight, give distance in track direction and orthogonal to it
          var straight = new RLine(this.lineStart.getStartPoint(), this.lineStart.getEndPoint());
          straight.rotate(Math.PI / 2, this.middleStart);
          straight.scale(new RVector(middleDiff * 4, middleDiff * 4), this.middleStart);
          var ortho = new RLine(this.lineEnd.getStartPoint(), this.lineEnd.getEndPoint());
          ortho.scale(new RVector(middleDiff * 4, middleDiff * 4), this.middleEnd);
          var isect = straight.getIntersectionPoints(ortho);
          if (isect.length == 1) {
            EAction.handleUserMessage(qsTr("distance in track direction: ") + this.middleStart.getDistanceTo(isect[0]).toFixed(1)
              + qsTr(", offset: ") + this.middleEnd.getDistanceTo(isect[0]).toFixed(1));
          }
        }
        this.setState(-1);
        this.terminate();
      }
      break;
    }
};

function getNormalizedAngle(a, limit, epsilon) {
    a = (Math.round(a / epsilon) % Math.round(limit / epsilon)) * epsilon;
    if (a < 0) {
      a += limit;
    }
    return a;
}

DistanceInfo.prototype.getAuxPreview = function() {
    var ret = undefined;
    if (isLineEntity(this.lineStart)) {
      ret = new Array();
      addAuxPreviewObjects(this.lineStart, ret);
      if (isLineEntity(this.lineEnd)) {
        addAuxPreviewObjects(this.lineEnd, ret);
        ret.push(new RLine(this.middleStart, this.middleEnd));
      }
    }
    return ret;
};

DistanceInfo.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.DistanceInfo, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setScriptFile(basePath + "/DistanceInfo.js");
    action.setDefaultShortcut(new QKeySequence("q,i"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.DistanceInfo);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
};

