/*
 * This script fills a straight distance with a track combination
 * by variegating of available tracks given in an user dialog.
 * This script highly re-uses the Variegate-script.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-05
 */

//http://www.ribbonsoft.com/doc/qcad/3.0/developer/class_r_entity.html

include("../QCTrack.js");
include("../Variegate/Variegate.js");
include("../../Modify/Transform.js");

function FillStraight(guiAction) {
    QCTrack.call(this, guiAction);
}

FillStraight.prototype = new Variegate();
FillStraight.includeBasePath = includeBasePath;

FillStraight.State = {
    SettingLineStart : 0,
    SettingLineEnd : 1
};

FillStraight.prototype.beginEvent = function() {
    EAction.prototype.beginEvent.call(this);
    var document = EAction.getDocument();

    this.setState(FillStraight.State.SettingLineStart);
};

FillStraight.prototype.setState = function(state) {
    Transform.prototype.setState.call(this, state);

    this.getDocumentInterface().setClickMode(RAction.PickEntity);
    this.setCrosshairCursor();

    var appWin = RMainWindowQt.getMainWindow();
    var info;
    switch (this.state) {
    case FillStraight.State.SettingLineStart:
      this.lineSource = undefined;
      this.lineDest = undefined;
      info = qsTr("Select start of straight line");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trCancel);
      break;
    case FillStraight.State.SettingLineEnd:
      this.lineDest = undefined;
      info = qsTr("Select end of straight line");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trBack);
      break;
    }
};

FillStraight.prototype.escapeEvent = function() {
    switch (this.state) {
    case FillStraight.State.SettingLineStart:
      Transform.prototype.escapeEvent.call(this);
      break;
    case FillStraight.State.SettingLineEnd:
      this.setState(FillStraight.State.SettingLineStart);
      break;
    }
};

FillStraight.prototype.pickEntity = function(event, preview) {
    var di = this.getDocumentInterface();
    var document = this.getDocument();
    var entityId = event.getEntityId();
    var entity = document.queryEntity(entityId);
    var pos = event.getModelPosition();

    switch (this.state) {
    case FillStraight.State.SettingLineStart:
      if (isNull(entity)) {
        this.lineStart = undefined;
        this.middleStart = undefined;
        this.lineEnd = undefined;
        this.middleEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineStart = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineStart == undefined) {
        this.middleStart = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.middleStart = this.lineStart.getMiddlePoints()[0];
      if (preview) {
        this.updatePreview();
      }
      else {
        this.setState(FillStraight.State.SettingLineEnd);
      }
      break;
    case FillStraight.State.SettingLineEnd:
      if (isNull(entity)) {
        this.lineEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineEnd = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineEnd == undefined) {
        this.middleEnd = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.middleEnd = this.lineEnd.getMiddlePoints()[0];
      if (preview) {
        this.updatePreview();
      }
      else {
        var op = this.getOperation(false);
        if (op == undefined) {
          break;
        }
        if (op == null) {
          this.setState(-1);
          this.terminate();
        }
        di.applyOperation(op);
        this.setState(-1);
        this.terminate();
      }
      break;
    }
};

FillStraight.prototype.getOperation = function(preview) {
    if (!isLineEntity(this.lineStart) || !isLineEntity(this.lineEnd)) {
      return undefined;
    }
    var angleDiff = getNormalizedAngle(this.lineEnd.getAngle() - this.lineStart.getAngle(), Math.PI, 0.001);
    var middleDiff = this.middleEnd.getDistanceTo(this.middleStart);
    var connectPossible = false, dist = 0, offset = 0;
    if (angleDiff > RMath.deg2rad(2)) { //too big difference, will not connect
      if (!preview) {
        EAction.handleUserMessage(qsTr("can not connect those track ends (angle differs too much: ") + RMath.rad2deg(angleDiff).toFixed(2) + qsTr("° > 2°)"));
      }
      return undefined;
    }
    var straight = new RLine(this.lineStart.getStartPoint(), this.lineStart.getEndPoint());
    straight.rotate(Math.PI / 2, this.middleStart);
    straight.scale(new RVector(middleDiff * 4, middleDiff * 4), this.middleStart);
    var ortho = new RLine(this.lineEnd.getStartPoint(), this.lineEnd.getEndPoint());
    ortho.scale(new RVector(middleDiff * 4, middleDiff * 4), this.middleEnd);
    var isect = straight.getIntersectionPoints(ortho);
    if (isect.length == 1) {
      dist = this.middleStart.getDistanceTo(isect[0]);
      offset = this.middleEnd.getDistanceTo(isect[0]);
    }
    else {
      if (!preview) {
        EAction.handleUserMessage(qsTr("can not connect those track ends (there's no intersection)"));
      }
      return undefined;
    }
    if (dist < 1) {
      if (!preview) {
        EAction.handleUserMessage(qsTr("can not connect those track ends (ends are too near)"));
      }
      return undefined;
    }
    var document = this.getDocument();
    var op = undefined;
    var targetLine;
    switch (this.lineStart.getSideOfPoint(this.middleEnd)) {
      case RS.LeftHand:
        targetLine = new RLine(this.middleStart, straight.getAngle(), dist);
        break;
      case RS.RightHand:
        targetLine = new RLine(this.middleStart, straight.getAngle() + Math.PI, dist);
        break;
      default:
        if (!preview) {
          EAction.handleUserMessage(qsTr("internal error: neither left nor right hand"));
        }
        return undefined;
    }
    if (preview) {
      op = new RAddObjectsOperation();
      op.addObject(new RLineEntity(document, new RLineData(this.middleStart, targetLine.getEndPoint())));
    }
    else {
      if (!this.doVariegation(targetLine)) {
        op = null;
      }
      else {
        op = new RAddObjectsOperation();
        this.targetPosition = this.middleStart;
        this.targetAngle = targetLine.getAngle();
        this.addBlockOperations(document, op, false);
      }
    }
    return op;
};

FillStraight.prototype.getAuxPreview = function() {
    var ret = undefined;
    if (isLineEntity(this.lineStart)) {
      ret = new Array();
      addAuxPreviewObjects(this.lineStart, ret);
      if (isLineEntity(this.lineEnd)) {
        addAuxPreviewObjects(this.lineEnd, ret);
      }
    }
    return ret;
};

FillStraight.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.FillStraight, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setScriptFile(basePath + "/FillStraight.js");
    action.setDefaultShortcut(new QKeySequence("q,f"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.FillStraight);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
};

