/*
 * This script moves + rotates the current selection so that
 * the track end selected by user connects to another track. The
 * original track is removed.
 * Attention! This script is highly re-used by DupMove, don't
 * change it without checking DupMove!
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-09
 */

include("../QCTrack.js");
include("../../Modify/Transform.js");

function Move(guiAction) {
    Transform.call(this, guiAction);

    this.lineSource = undefined;
    this.lineDest = undefined;
}

Move.prototype = new Transform();
Move.includeBasePath = includeBasePath;

Move.State = {
    SettingLineSource : 0,
    SettingLineDest : 1
};

Move.prototype.beginEvent = function() {
    Transform.prototype.beginEvent.call(this);

    if (!this.verifySelection()) {
        return;
    }

    this.continueAfterMove = false;
    this.copies = 0; //delete original
    this.useCurrentAttributes = false; //do not use current attributes
    this.forceNewObject = false;
    this.setState(Move.State.SettingLineSource);
};

Move.prototype.setState = function(state) {
    Transform.prototype.setState.call(this, state);

    this.getDocumentInterface().setClickMode(RAction.PickEntity);
    this.setCrosshairCursor();

    var appWin = RMainWindowQt.getMainWindow();
    var info;
    switch (this.state) {
    case Move.State.SettingLineSource:
      this.lineSource = undefined;
      this.lineDest = undefined;
      info = qsTr("Select end of track to move");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trCancel);
      break;
    case Move.State.SettingLineDest:
      this.lineDest = undefined;
      info = qsTr("Select destination track end");
      appWin.setCommandPrompt(info);
      appWin.setLeftMouseTip(info);
      appWin.setRightMouseTip(EAction.trBack);
      break;
    }
};

Move.prototype.escapeEvent = function() {
    switch (this.state) {
    case Move.State.SettingLineSource:
      Transform.prototype.escapeEvent.call(this);
      break;
    case Move.State.SettingLineDest:
      this.setState(Move.State.SettingLineSource);
      break;
    }
};

Move.prototype.pickEntity = function(event, preview) {
    var di = this.getDocumentInterface();
    var document = this.getDocument();
    var entityId = event.getEntityId();
    var entity = document.queryEntity(entityId);
    var pos = event.getModelPosition();
    var op;

    switch (this.state) {
    case Move.State.SettingLineSource:
      if (isNull(entity)) {
        this.lineSource = undefined;
        this.lineDest = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineSource = getClosestTrackEnd(document, entity, pos, true);
      if (this.lineSource == undefined) {
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      if (preview) {
        this.updatePreview();
      }
      else {
        this.setState(Move.State.SettingLineDest);
      }
      break;
    case Move.State.SettingLineDest:
      if (isNull(entity)) {
        this.lineDest = undefined;
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      this.lineDest = getClosestTrackEnd(document, entity, pos, false);
      if (this.lineDest == undefined) {
        if (preview) {
            this.updatePreview();
        }
        break;
      }
      if (preview) {
        this.updatePreview();
      }
      else {
        var oldSelection = document.querySelectedEntities();
        op = this.getOperation(false);
        if (op == undefined) {
          break;
        }
        di.applyOperation(op);
        if (this.continueAfterMove) { //this is needed for DupMove
          di.selectEntities(oldSelection);
          this.lineDest = undefined;
          this.updatePreview();
        }
        else { //this is for normale move
          this.setState(-1);
          this.terminate();
        }
      }
      break;
    }
};

Move.prototype.getOperation = function(preview) {
    if (!isLineEntity(this.lineSource) || !isLineEntity(this.lineDest)) {
      return undefined;
    }

    //see http://www.ribbonsoft.com/doc/qcad/3.0/developer/class_r_vector.html
    var moveX, moveY;
    moveX = this.lineDest.getEndPoint().getX() - this.lineSource.getStartPoint().getX();
    moveY = this.lineDest.getEndPoint().getY() - this.lineSource.getStartPoint().getY();
    this.moveBy = new RVector(moveX, moveY);
    this.angle = RMath.getNormalizedAngle(this.lineDest.getAngle() - this.lineSource.getAngle() + Math.PI);
    if (!preview) {
      EAction.handleUserMessage(qsTr("move by ") + moveX + qsTr(" / ") + moveY + qsTr(", rotate by ") + RMath.rad2deg(this.angle) + qsTr("°"));
    }
    return Transform.prototype.getOperation.call(this, preview);
};

Move.prototype.transform = function(entity, k, op) {
    entity.move(this.moveBy);
    entity.rotate(this.angle, this.lineDest.getEndPoint());
    op.addObject(entity, this.useCurrentAttributes, this.forceNewObject);
};

Move.prototype.getAuxPreview = function() {
    var ret = undefined;
    if (isLineEntity(this.lineSource)) {
      ret = new Array();
      addAuxPreviewObjects(this.lineSource, ret);
    }
    return ret;
};

Move.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.Move, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setRequiresSelection(true);
    action.setScriptFile(basePath + "/Move.js");
    action.setIcon(basePath + "/Move.svg");
    action.setDefaultShortcut(new QKeySequence("q,m"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.Move);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
    var appWin = EAction.getMainWindow();
    appWin.addSelectionListener(action);
};

