/*
 * This script counts selected (if none selected: all) tracks and
 * prints a summary in the user window.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-05
 */

include("../QCTrack.js");

function PartList(guiAction) {
    EAction.call(this, guiAction);
}

PartList.prototype = new EAction();
PartList.includeBasePath = includeBasePath;

PartList.prototype.beginEvent = function() {
    var document = EAction.getDocument();

    var dialog = WidgetFactory.createWidget(
                   "scripts/QCTrack/PartList",
                   "PartListDialog.ui");
    var widgets = getWidgets(dialog);
    var checkIds = document.querySelectedEntities();
    if (checkIds.length == 0) {
      widgets["entAllBlocks"].checked = true;
      widgets["grpEnt"].enabled = false;
    }
    if (!dialog.exec()) { //the user aborted
        this.terminate();
        return;
    }
    var useNumbersInsideBlock = widgets["nameNumber"].checked;
    var evalCost = widgets["nameBlockWithCost"].checked;
    var filestream = undefined;
    var file;
    if (widgets["toFile"].checked) {
      var fileName = getFileNameToSaveFromUser(qsTr("Part List File"), "TXT", ".txt");
      if (fileName == undefined) {
        this.terminate();
        return;
      }
      file = new QFile(fileName);
      var flags = new QIODevice.OpenMode(QIODevice.WriteOnly | QIODevice.Text);
      if (!file.open(flags)) {
          this.terminate();
          return;
      }
      filestream = new QTextStream(file);
    }
    if (widgets["entAllBlocks"].checked) {
      checkIds = document.queryAllBlockReferences();
    }
    var list = new Array(); //each element contains a three-field array containing the track number, the current amount and the current length and the current cost of this track type
    var blockCount = 0, nmbrCount = 0, costSum = 0, lengthSum = 0;
    var unhandledAlreadyPrinted = false;

    for (var i = 0; i < checkIds.length; i++) { //first run: count all blocks with TR_*
      var id = checkIds[i];
      var block = document.queryEntity(id);
      if (!isBlockReferenceEntity(block)) {
        continue;
      }
      blockCount++;
      if (useNumbersInsideBlock) {
        var data = block.getData();
        var subIds = document.queryBlockEntities(data.getReferencedBlockId());
        for (var j = 0; j < subIds.length; j++) {
          var text = data.queryEntity(subIds[j]);
          if (isTextEntity(text) && text.getLayerName() == "TR_NMBRS") {
            var nmbr = text.getPlainText();
            if (nmbr == undefined || nmbr == "") {
              continue;
            }
            nmbrCount++;
            inc(list, nmbr, 1, 0, 0);
          }
        }
      }
      else {
        var blockName = block.getReferencedBlockName();
        inc(list, blockName, 1, 0, 0);
        var data = block.getData();
        var subIds = document.queryBlockEntities(data.getReferencedBlockId());
        for (var j = 0; j < subIds.length; j++) {
          var sub = data.queryEntity(subIds[j]);
          if (sub.getLayerName() == "TR_MIDS") {
            var len = 0;
            if (isLineEntity(sub)) {
              len = sub.getStartPoint().getDistanceTo(sub.getEndPoint());
            }
            else if (isArcEntity(sub)) {
              len = sub.getAngleLength() * sub.getRadius();
            }
            else if (!unhandledAlreadyPrinted) {
              EAction.handleUserMessage("block " + blockName + " containts unhandled entity on TR_MIDS, ignoring them (message will be printed only once)");
              unhandledAlreadyPrinted = true;
            }
            inc(list, blockName, 0, len, 0);
            lengthSum += len;
          }
          if (isTextEntity(sub) && evalCost && sub.getLayerName() == "TR_COST") {
            var addCost = parseInt(sub.getPlainText());
            if (addCost > 0) {
              inc(list, blockName, 0, 0, addCost);
              costSum += addCost;
            }
          }
        }
      }
    }
    list.sort(function(a, b) {
      return a[0].localeCompare(b[0]);
    });
    if (useNumbersInsideBlock) {
      print(filestream, qsTr("processed ") + blockCount + qsTr(" blocks containing ") + nmbrCount + qsTr(" numbers:"));
      for (var i = 0; i < list.length; i++) { //second run: print track numbers and track count
        print(filestream, list[i][0] + ": " + list[i][1]);
      }
    }
    else {
      print(filestream, qsTr("processed ") + blockCount + qsTr(" blocks:"));
      for (var i = 0; i < list.length; i++) { //second run: print track numbers and track count
        print(filestream, list[i][0] + ": " + list[i][1] + ": len: " + list[i][2].toFixed(1) + (evalCost ? ": cost: " + (list[i][3] / 100.0).toFixed(2) : ":"));
      }
      print(filestream, qsTr("length of all blocks: ") + lengthSum.toFixed(1));
      if (evalCost) {
        print(filestream, qsTr("cost of all blocks: ") + (costSum / 100.0).toFixed(2));
      }
    }

    if (filestream !== undefined) {
      file.close();
    }
    this.terminate();
};

function print(filestream, what) {
    if (filestream == undefined) {
      EAction.handleUserMessage(what);
    }
    else {
      filestream.writeString(what + "\n");
    }
}

function inc(list, name, by1, by2, by3) {
  var k = 0;
  while (k < list.length) {
    if (list[k][0] == name) {
      list[k][1] += by1;
      list[k][2] += by2;
      list[k][3] += by3;
      break;
    }
    k++;
  }
  if (k == list.length) {
    list.push([name, by1, by2, by3]);
  }
}

PartList.init = function(basePath) {
    var action = new RGuiAction(QCTrack.MenuItem.PartList, RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setScriptFile(basePath + "/PartList.js");
    action.setIcon(basePath + "/PartList.svg");
    action.setDefaultShortcut(new QKeySequence("q,p"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.PartList);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
    var appWin = EAction.getMainWindow();
    appWin.addSelectionListener(action);
};

