/*
 * This file leads to a track list widget in QCad.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2015-01-15
 */

include("../QCTrack.js");
include("../../WidgetFactory.js");

/**
 * Track list widget class (QWidget).
 */
function RTrackListQt(parent, otherWidgets) {
    RListWidget.call(this, parent);
    this.di = null;
    this.list = undefined;
    this.otherWidgets = otherWidgets;

    var appWin = EAction.getMainWindow();
    var adapter = new RBlockListenerAdapter();
    appWin.addBlockListener(adapter);
    adapter.blocksUpdated.connect(this, "updateTracks");
    adapter.blocksCleared.connect(this, "clearTracks");

    this.otherWidgets["showTrackIcons"]["clicked()"].connect(this, function() { this.updateTracks(EAction.getDocumentInterface()); });
    this.itemSelectionChanged.connect(this, "trackActivated");
    this.itemDoubleClicked.connect(this, "insertTrack");
    this.basePath = includeBasePath;
}

RTrackListQt.prototype = new RListWidget();

RTrackListQt.prototype.contextMenuEvent = function(e) {
    var item = this.itemAt(e.pos());
    if (!isNull(item)) {
        this.setCurrentItem(item);
    }
    this.rotateTrack();
    e.ignore();
};

/**
 * Called when blocks are addded, edited or deleted. Updates the track list.
 */
RTrackListQt.prototype.updateTracks = function(documentInterface) {
    this.di = documentInterface;

    var block;
    
    var currentRow = this.currentRow;
    var pos = this.verticalScrollBar().sliderPosition;
    this.clear();
    if (isNull(documentInterface)) {
        return;
    }

    var document = documentInterface.getDocument();
    this.clear();
    this.list = new Array(); //contains a five element array per track, each 0: block name, 1: block id, 2: array with track ends, 3: radius as text, 4: length/angle as text
    var allIds = document.queryAllBlocks();
    for (var i = 0; i < allIds.length; i++) { //first run: collect blocks with exactly one straight line in TR_MIDS
      var id = allIds[i];
      var block = document.queryBlockDirect(id);
      var blockName = block.getName();
      if (blockName[0] == '*') { //reserved blocks like paper space
        continue;
      }
      var k = 0;
      while (k < this.list.length) {
        if (this.list[k][0] == blockName) { //block already existing
          break;
        }
        k++;
      }
      if (k == this.list.length) {
        var subIds;
        subIds = document.queryBlockEntities(id);
        var blockLen = undefined;
        var blockRadius = undefined;
        var blockEnds = new Array();
        for (var j = 0; j < subIds.length; j++) {
          var sub = document.queryEntity(subIds[j]);
          if (sub.getLayerName() == "TR_MIDS") {
            if (blockLen != undefined) { //block has another entity on TR_MIDS layer
              blockLen = "";
              blockRadius = qsTr("complex");
            }
            else if (isLineEntity(sub)) {
              blockLen = sub.getStartPoint().getDistanceTo(sub.getEndPoint()).toFixed(1);
              blockRadius = "";
            }
            else if (isArcEntity(sub)) {
              blockLen = RMath.rad2deg(sub.getAngleLength()).toFixed(2) + "°";
              blockRadius = sub.getRadius().toFixed(1);
            }
          }
          else if (sub.getLayerName() == "TR_ENDS") {
            blockEnds.push(sub);
          }
        }
        if (blockLen != undefined && blockEnds.length > 0) {
          this.list.push([blockName, id, blockEnds, blockRadius, blockLen]);
        }
      }
    }
    if (this.list.length != 0) {
      this.list.sort(function(a, b) {
        return a[0].localeCompare(b[0]);
      });
      var curveColors = new Array(); //will be used only inside generateTrackTrackPreview
      for (var i = 0; i < this.list.length; i++) {
        var w = new QWidget();
        var vbl = new QHBoxLayout(w);
        var item;
        if (this.otherWidgets["showTrackIcons"].checked) {
          vbl.addWidget(item = new QLabel(""), 0, 0);
          item.setPixmap(generateTrackPreview(document, this.list[i][1], curveColors, 32));
        }
        vbl.addWidget(item = new QLabel(this.list[i][0]), 0, 0);
        vbl.addWidget(item = new QLabel(this.list[i][3]), 0, 0); 
        item.alignment = Qt.AlignVCenter | Qt.AlignRight;
        vbl.addWidget(item = new QLabel(this.list[i][4]), 0, 0); 
        item.alignment = Qt.AlignVCenter | Qt.AlignRight;
        w.setLayout(vbl);
        var lwi = new QListWidgetItem(this);
        lwi.setSizeHint(w.sizeHint);
        this.addItem(lwi);
        lwi.setToolTip(this.list[i][0]); //this is used in TrackWidgetInsert to get the block name
        this.setItemWidget(lwi, w);
      }
    }

    this.verticalScrollBar().sliderPosition = pos;
    if (currentRow!=-1 && !isNull(this.item(currentRow))) {
        this.currentRow = currentRow;
        this.item(currentRow).setSelected(true);
    }
    this.trackActivated();
};

/**
 * Called when no document is active. Clears the track list.
 */
RTrackListQt.prototype.clearTracks = function() {
    this.clear();
    this.enableActions(false);
};

/**
 * Called when a track name is activated (clicked, highlighted) in the track list.
 */
RTrackListQt.prototype.trackActivated = function() {
    if (isNull(this.di)) {
        this.enableActions(false);
        return;
    }

    var list = this.selectedItems();
    if (list.length===0) {
        this.enableActions(false);
        return;
    }

    var document = this.di.getDocument();
    if (isNull(document)) {
        this.enableActions(false);
        return;
    }

    var ind = this.currentRow;
    var connEnd = this.otherWidgets["connEnd"];
    var oldConn = connEnd.currentIndex;
    connEnd.clear();
    if (ind >= 0 && ind < this.list.length) {
      for (var i = 0; i < this.list[ind][2].length; i++) {
        connEnd.addItem("" + (i + 1));
      }
    }
    if (connEnd.count > 0) {
      if (oldConn < 0 || oldConn >= connEnd.count) {
        connEnd.currentIndex = 0;
      }
      else {
        connEnd.currentIndex = oldConn;
      }
    }
    this.enableActions(true);
};

RTrackListQt.prototype.enableActions = function(enable) {
    var override;
    if (enable===true) {
      override = 1;
    }
    else {
      override = 0;
    }
    this.otherWidgets["addTrack"].enabled = enable;
    this.otherWidgets["rotateTrack"].enabled = enable;
    this.otherWidgets["connEnd"].enabled = enable;
    var action;
    action = RGuiAction.getByScriptFile("scripts/QCTrack/TrackWidgetInsert/TrackWidgetInsert.js");
    action.setEnabledOverride(enable, override);
    action = RGuiAction.getByScriptFile("scripts/QCTrack/TrackWidgetRotate/TrackWidgetRotate.js");
    action.setEnabledOverride(enable, override);
};

RTrackListQt.prototype.insertTrack = function() {
    var action = RGuiAction.getByScriptFile("scripts/QCTrack/TrackWidgetInsert/TrackWidgetInsert.js");
    if (!isNull(action)) {
        action.slotTrigger();
    }
};

RTrackListQt.prototype.rotateTrack = function() {
    var action = RGuiAction.getByScriptFile("scripts/QCTrack/TrackWidgetRotate/TrackWidgetRotate.js");
    if (!isNull(action)) {
        action.slotTrigger();
    }
};


/**
 * \class TrackWidget
 * \brief Initializes the block list widget (RTrackListQt).
 * \ingroup ecma_widgets
 */
function TrackWidget(guiAction) {
    Widgets.call(this, guiAction);
}

TrackWidget.prototype = new Widgets();

TrackWidget.getPreferencesCategory = function() {
    return [ qsTr("Widgets"), qsTr("Track Widget") ];
};

TrackWidget.applyPreferences = function(doc, mdiChild) {
    var appWin = RMainWindowQt.getMainWindow();
    appWin.notifyBlockListeners(EAction.getDocumentInterface());
};

TrackWidget.prototype.clearTracks = function() {
    this.trackTable.clear();
};

/**
 * Shows / hides the track list.
 */
TrackWidget.prototype.beginEvent = function() {
    Widgets.prototype.beginEvent.call(this);

    var appWin = RMainWindowQt.getMainWindow();
    var dock = appWin.findChild("TrackWidgetDock");
    if (!QCoreApplication.arguments().contains("-no-show")) {
        dock.visible = !dock.visible;
    }
};

TrackWidget.prototype.finishEvent = function() {
    Widgets.prototype.finishEvent.call(this);

    var appWin = RMainWindowQt.getMainWindow();
    var dock = appWin.findChild("TrackWidgetDock");
    this.getGuiAction().setChecked(dock.visible);
};


/**
 * Initializes the track list widget.
 */
TrackWidget.init = function(basePath) {
    var appWin = RMainWindowQt.getMainWindow();

    var action = new RGuiAction(QCTrack.MenuItem.TrackWidget, appWin);
    action.setRequiresDocument(false);
    action.setScriptFile(basePath + "/TrackWidget.js");
    action.setIcon(basePath + "/TrackWidget.svg");
    action.setDefaultShortcut(new QKeySequence("q,v"));
    action.setGroupSortOrder(QCTrack.MenuOrder.GroupSortOrder);
    action.setSortOrder(QCTrack.MenuOrder.TrackWidget);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);

    var formWidget = WidgetFactory.createWidget(basePath, "TrackWidget.ui");
    var widgets = getWidgets(formWidget);
    var layout = formWidget.findChild("verticalLayout");
    this.trackList = new RTrackListQt(layout, widgets);
    this.trackList.objectName = "TrackList";
    widgets["verticalLayout"].addWidget(this.trackList, 1, 0);
    widgets["addTrack"]["clicked()"].connect(this, function() { this.trackList.insertTrack(); });
    widgets["rotateTrack"]["clicked()"].connect(this, function() { this.trackList.rotateTrack(); });

    var dock = new RDockWidget(qsTr("Track Widget"), appWin);
    dock.objectName = "TrackWidgetDock";
    dock.setWidget(formWidget);
    appWin.addDockWidget(Qt.RightDockWidgetArea, dock);

    dock.shown.connect(function() { action.setChecked(true); });
    dock.hidden.connect(function() { action.setChecked(false); });

    dock.visible = false;
};

